<?php
namespace Indeterminado\Repositories;

class Receitabruta extends \Doctrine\ORM\EntityRepository
{


    /**
     * Atualiza receita bruta de uma determinada empresa.
     *
     * @param $empresa
     * @param $primeiroDiaMesDtProc
     * @param $ultimoDiaMesDtProc
     * @param $nrorg
     *
     * @return \Doctrine\DBAL\Driver\Statement
     */
    public function atualizaReceita($empresa, $primeiroDiaMesDtProc, $ultimoDiaMesDtProc, $nrorg){
        $updt = "UPDATE RECEITABRUTA
                    SET VRRECEITABR    = NVL(
                                            (SELECT SUM(VRNOTAFISC) AS VRNOTAFISC
                                               FROM NOTAFISCAL N,
                                                    (SELECT CDTIPOOPER,
                                                            IDENTRSAIDOP,
                                                            NRORG
                                                       FROM TIPOOPERAC
                                                      WHERE NRORG         = :P_NRORG
                                                        AND IDUTRECEITA = 'S'
                                                        AND IDENTRSAIDOP  = 'S'
                                                    ) T
                                              WHERE N.NRORG        = :P_NRORG
                                                AND N.CDEMPRESA    = :P_CDEMPRESA
                                                AND N.NRORG        = T.NRORG
                                                AND N.CDTIPOOPER   = T.CDTIPOOPER
                                                AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                                                AND N.DTENTRSAID BETWEEN :P_PRIMEIRODIA AND :P_ULTIMODIA
                                                AND N.IDSTATUS IN ('E','A')
                                                AND NVL(N.IDSISTEMA, 'DFS') <>  'ITG'
                                                AND NVL(N.IDSISTEMA, 'DFS') <>  'REC'
                                            ), VRRECEITABR),
                                        IDSTATUSMES = 'B'
                  WHERE NRORG          = :P_NRORG
                    AND CDEMPRESA      = :P_CDEMPRESA
                    AND DTREFRECEITABR = :P_PRIMEIRODIA";

        $params = array(
            'P_CDEMPRESA' => $empresa,
            'P_PRIMEIRODIA' => $primeiroDiaMesDtProc,
            'P_ULTIMODIA' => $ultimoDiaMesDtProc,
            'P_NRORG' => $nrorg,
        );

        $em = \Setforms2\Shared\Registry::getEntityManager();
        return $em->getConnection()->executeQuery($updt, $params);
    }


    /**
     * Consulta um registro referente à receita bruta da respectiva empresa.
     *
     * @param $nrorg
     * @param $empresa
     * @param $dtReferencia
     *
     * @return mixed
     */
    public function consultaReceitaBrutaEmpresa($nrorg, $empresa, $dtReferencia){
        $sql = "SELECT R.*
                  FROM RECEITABRUTA R
                 WHERE R.NRORG                                       = :P_NRORG
                   AND R.CDEMPRESA                                   = :P_CDEMPRESA
                   AND TO_CHAR(TO_DATE(R.DTREFRECEITABR), 'MM/YYYY') = TO_CHAR(TO_DATE(:P_DTREFRECEITABR), 'MM/YYYY')";

        $params = array(
            'P_NRORG'     => $nrorg,
            'P_CDEMPRESA' => $empresa,
            'P_DTREFRECEITABR' => $dtReferencia,
        );

        $this->_em->getConnection()->executeQuery($sql, $params);
        $entityManager = $this->getEntityManager();
        $resultSetMapping = new \Doctrine\ORM\Query\ResultSetMappingBuilder($entityManager);
        $resultSetMapping->addRootEntityFromClassMetadata('Indeterminado\Entities\Receitabruta', "R");

        $result = $entityManager->createNativeQuery($sql, $resultSetMapping)->execute($params);
        return count($result) > 0 ? $result : false;
    }


    //Consulta o valor do saldo negativo do mes anterior
    public static function consultaSaldoNegatMesAnt($cdempresa, $cdsimplesnac, $dtrefreceitabr) {

        $sql = "SELECT NVL(VRSALDONEGMES, 0) VRSALDONEGMES
                  FROM RECEITABRUTA
                 WHERE CDEMPRESA = :CDEMPRESA
                   AND CDSIMPLESNACIONAL = :CDSIMPLESNACIONAL
                   AND DTREFRECEITABR = (SELECT  MAX(DTREFRECEITABR)
                                           FROM RECEITABRUTA
                                          WHERE CDEMPRESA = :CDEMPRESA
                                            AND CDSIMPLESNACIONAL = :CDSIMPLESNACIONAL
                                            AND DTREFRECEITABR < TO_DATE(:DTREFRECEITABR, 'DD/MM/YYYY hh24:mi:ss')
                                         )";

        $params = array('CDEMPRESA'         => $cdempresa,
                        'CDSIMPLESNACIONAL' => $cdsimplesnac,
                        'DTREFRECEITABR'    => $dtrefreceitabr);

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    //Consulta o valor da receita atraves das notas fiscais, considerando o que compoe e o que deduz
    public static function consultaValorReceitaCompDed($cdempresa, $cdsimplesnac, $dtrefreceitabr, $tiporeceita) {

        $sql = "SELECT SUM(NVL(VRNOTAFISC,0)) AS VRNOTAFISC
                  FROM (
                       SELECT SUM(NVL(I.VRBRUTITEM,0) - NVL(VALOR_IMPOSTO_ITEM_NFE(I.CDFILIAL, I.NRLANCTONF, I.NRSEQITPROD,'80','I','1'),0)) AS VRNOTAFISC
                         FROM ITEMPRODUTO I, NOTAFISCAL N,
                              (SELECT CDTIPOOPER, IDENTRSAIDOP
                                 FROM TPOPERSIMPLESNAC
                                WHERE IDTIPOOPERREC = :IDTIPOOPERREC
                                  AND CDSIMPLESNACIONAL = :CDSIMPLESNACIONAL
                              ) T
                        WHERE N.CDEMPRESA  = :CDEMPRESA
                          AND N.CDTIPOOPER   = T.CDTIPOOPER
                          AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                          AND N.DTENTRSAID BETWEEN TRUNC(TO_DATE(:DTREFRECEITABR), 'MONTH') AND LAST_DAY(TO_DATE(:DTREFRECEITABR))
                          AND N.IDENTRSAIDOP = 'E'
                          AND N.IDSTATUS              IN ('E','A')
                          AND NVL(N.IDSISTEMA, 'DFS') <> 'ITG'
                          AND NVL(N.IDSISTEMA, 'DFS') <> 'REC'
                          AND NVL(N.IDSISTEMA, 'DFS') <> 'PAG'
                          AND NVL(N.IDSISTEMA, 'DFS') <> 'TMP'
                          AND I.CDFILIAL     = N.CDFILIAL
                          AND I.NRLANCTONF   = N.NRLANCTONF
                          AND I.CDSIMPLESNACIONAL = :CDSIMPLESNACIONAL
                    UNION ALL
                   SELECT SUM(NVL(I.VRBRUTITEM,0) - NVL(VALOR_IMPOSTO_ITEM_NFE(I.CDFILIAL, I.NRLANCTONF, I.NRSEQITPROD,'80','I','1'),0)) AS VRNOTAFISC
                         FROM ITEMPRODUTO I, NOTAFISCAL N,
                              (SELECT CDTIPOOPER, IDENTRSAIDOP
                                 FROM TPOPERSIMPLESNAC
                                WHERE IDTIPOOPERREC = :IDTIPOOPERREC
                                  AND CDSIMPLESNACIONAL = :CDSIMPLESNACIONAL
                              ) T
                        WHERE N.CDEMPRESA  = :CDEMPRESA
                          AND N.CDTIPOOPER   = T.CDTIPOOPER
                          AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                          AND N.DTEMISSAO BETWEEN TRUNC(TO_DATE(:DTREFRECEITABR), 'MONTH') AND LAST_DAY(TO_DATE(:DTREFRECEITABR))
                          AND N.IDENTRSAIDOP = 'S'
                          AND N.IDSTATUS              IN ('E','A')
                          AND NVL(N.IDSISTEMA, 'DFS') <> 'ITG'
                          AND NVL(N.IDSISTEMA, 'DFS') <> 'REC'
                          AND NVL(N.IDSISTEMA, 'DFS') <> 'PAG'
                          AND NVL(N.IDSISTEMA, 'DFS') <> 'TMP'
                          AND I.CDFILIAL     = N.CDFILIAL
                          AND I.NRLANCTONF   = N.NRLANCTONF
                          AND I.CDSIMPLESNACIONAL = :CDSIMPLESNACIONAL
                          AND ((LENGTH(I.CDFISCOPERSN) = 3 AND I.CDFISCOPERSN <> '400') OR
                              (LENGTH(I.CDFISCOPERSN) = 4 AND NVL(RTRIM(SUBSTR(I.CDFISCOPERSN,2,3)),'0') <> '400'))
                    UNION ALL
                       SELECT NVL(SUM(VRNOTAFISC), 0) AS VRNOTAFISC
                         FROM NOTAFISCAL N,
                              (SELECT CDTIPOOPER, IDENTRSAIDOP
                                 FROM TPOPERSIMPLESNAC
                                WHERE IDTIPOOPERREC = :IDTIPOOPERREC
                                  AND CDSIMPLESNACIONAL = :CDSIMPLESNACIONAL
                              ) T
                        WHERE N.CDEMPRESA  = :CDEMPRESA
                          AND N.CDTIPOOPER   = T.CDTIPOOPER
                          AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                          AND N.DTENTRSAID BETWEEN TRUNC(TO_DATE(:DTREFRECEITABR), 'MONTH') AND LAST_DAY(TO_DATE(:DTREFRECEITABR))
                          AND N.IDSTATUS              IN ('E','A')
                          AND NVL(N.IDSISTEMA, 'DFS') = 'FIS'
                   )";

        $params = array('CDEMPRESA'         => $cdempresa,
                        'CDSIMPLESNACIONAL' => $cdsimplesnac,
                        'DTREFRECEITABR'    => $dtrefreceitabr,
                        'IDTIPOOPERREC'     => $tiporeceita);

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    //Consulta se existem meses posteriores ao mes e, se existe, busca esses registros
    public static function consultaMesesPost($cdempresa, $cdsimplesnac, $dtrefreceitabr, $ano) {

        $sql = "SELECT CDEMPRESA, DTREFRECEITABR, CDSIMPLESNACIONAL
                  FROM RECEITABRUTA
                 WHERE CDEMPRESA = :CDEMPRESA
                   AND CDSIMPLESNACIONAL = :CDSIMPLESNACIONAL
                   AND TO_DATE(TO_CHAR(DTREFRECEITABR, 'DD/MM/YYYY'), 'DD/MM/YYYY') > TO_DATE(TO_CHAR(TO_DATE(:DTREFRECEITABR, 'DD/MM/YYYY hh24:mi:ss'), 'DD/MM/YYYY'), 'DD/MM/YYYY')
              ORDER BY DTREFRECEITABR
              ";

        $params = array('CDEMPRESA'         => $cdempresa,
                        'CDSIMPLESNACIONAL' => $cdsimplesnac,
                        'DTREFRECEITABR'    => $dtrefreceitabr);

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    //Consulta as notas fiscais que foram utilizadas para compor ou deduzir a receita bruta
    public static function consultaNFCalcRecBrut($cdempresa, $cdsimplesnac, $dtrefreceitabr) {
        $sql = "SELECT CDFILIAL, NRLANCTONF
                  FROM NOTAFISCAL N,
                       (SELECT CDTIPOOPER,IDENTRSAIDOP
                          FROM TPOPERSIMPLESNAC
                         WHERE CDSIMPLESNACIONAL = :CDSIMPLESNACIONAL
                           AND IDTIPOOPERREC IN ('C','D')
                       ) T
                 WHERE N.CDEMPRESA  = :CDEMPRESA
                   AND N.CDTIPOOPER   = T.CDTIPOOPER
                   AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                   AND N.DTENTRSAID BETWEEN TRUNC(TO_DATE(:DTREFRECEITABR), 'MONTH') AND LAST_DAY(TO_DATE(:DTREFRECEITABR))
                   AND N.IDSTATUS IN ('E','A')";

        $params = array('CDEMPRESA'         => $cdempresa,
                        'CDSIMPLESNACIONAL' => $cdsimplesnac,
                        'DTREFRECEITABR'    => $dtrefreceitabr);

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    //Grava para as notas fiscais que entraram no calculo da receita bruta a data do mes que foi calculado
//    public static function setaDataNFCalcRecBrut($cdfilialNotaF, $nrNotaF, $dtrefreceitabr) {
//        $sql = "UPDATE NOTAFISCAL
//                   SET DTCALCRECBRUTA = TRUNC(TO_DATE(:DTREFRECEITABR), 'MONTH')
//                 WHERE CDFILIAL   = :CDFILIAL
//                   AND NRLANCTONF = :NRLANCTONF";
//
//        $params = array('CDFILIAL'       => $cdfilialNotaF,
//                        'NRLANCTONF'     => $nrNotaF,
//                        'DTREFRECEITABR' => $dtrefreceitabr);
//
//        try{
//            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
//        } catch(\Exception $e) {
//            throw $e;
//        }
//    }


}